@@ -12,7 +12,7 @@ from pyrogram import Client
 
 from . import generals
 from .cog import Cog
-from .objects import Message, ChatActions, UserProfilePicture, CallbackQuery, Command
+from .objects import Message, ChatActions, UserProfilePicture, CallbackQuery, Command, ChatPermission, PromotePermission
 
 
 class CallbackError(Exception):
@@ -24,14 +24,28 @@ class CallbackError(Exception):
             super().__init__(*args)
 
 
+class ToMenyRequests(Exception):
+    def __init__(self, message=None, *args):
+        if message is not None:
+            self.message = message
+            super().__init__(message, *args)
+        else:
+            super().__init__(*args)
+
+
+class TgAPIException(Exception):
+    def __init__(self, message=None, *args):
+        if message is not None:
+            self.message = message
+            super().__init__(message, *args)
+        else:
+            super().__init__(*args)
+
+
 class Bot:
 
     def __init__(self, prefixs, user_id, user_hash, **kwargs):
-        try:
-            self.pyrogram = Client('me', user_id, user_hash).start()
-            logging.basicConfig(level=logging.INFO)
-        except Exception:
-            pass
+        self.pyrogram = Client('me', user_id, user_hash).start()
         self.random_cog = None
         self.url = ''
         self.token = ''
@@ -92,6 +106,81 @@ class Bot:
     async def send_chat_action(self, chat_id, chat_action: ChatActions):
         return await self._tg_request('sendChatAction', True, **{'chat_id': chat_id, 'action': chat_action.value})
 
+    async def send_dice(self, chat_id, emoji, **kwargs):
+        dic = {
+            'chat_id': chat_id,
+            'emoji': emoji
+        }
+        dic.update(kwargs)
+        rs = await self._tg_request('sendDice', True, **dic)
+        if not rs.get('ok'):
+            if rs.get('error_code') == 429:
+                raise ToMenyRequests(message=None, *rs)
+        return rs.get('result').get('dice').get('value')
+
+    async def restrict_chat_member(self, chat_id: int, user_id: int, permission: ChatPermission, until_date=None):
+        dic = {
+            'chat_id': chat_id,
+            'user_id': user_id,
+            'permission': permission.to_dict
+        }
+        if until_date:
+            dt = datetime.datetime.now() + datetime.timedelta(seconds=int(30))
+            unix_time = time.mktime(dt.timetuple())
+            dic.update({'until_date': unix_time})
+        rs = await self._tg_request('restrictChatMember', True, **dic)
+        if not rs.get('ok'):
+            if rs.get('error_code') == 429:
+                raise ToMenyRequests(rs.get('description'), rs)
+            else:
+                raise TgAPIException(rs)
+        return rs
+
+    async def promote_chat_member(self, chat_id: int, user_id: int, promotePermission: PromotePermission):
+        dic = {
+            'chat_id': chat_id,
+            'user_id': user_id
+        }
+        dic.update(promotePermission.to_dict)
+        rs = await self._tg_request('promoteChatMember', True, **dic)
+        if not rs.get('ok'):
+            if rs.get('error_code') == 429:
+                raise ToMenyRequests(rs.get('description'), rs)
+            else:
+                raise TgAPIException(rs)
+        return rs
+
+    async def set_chat_administrator_custom_title(self, chat_id: int, user_id: int, custom_title: str):
+        dic = {
+            'chat_id': chat_id,
+            'user_id': user_id,
+            'custom_title': custom_title
+        }
+        rs = await self._tg_request('setChatAdministratorCustomTitle', True, **dic)
+        if not rs.get('ok'):
+            if rs.get('error_code') == 429:
+                raise ToMenyRequests(rs.get('description'), rs)
+            else:
+                raise TgAPIException(rs)
+        return rs
+
+    async def create_forum_topic(self, chat_id: int, name: str, icon_color=None, icon_custom_emoji_id=None):
+        dic = {
+            'chat_id': chat_id,
+            'name': name
+        }
+        if icon_color:
+            dic.update({'icon_color': icon_color})
+        if icon_custom_emoji_id:
+            dic.update({'icon_custom_emoji_id': icon_custom_emoji_id})
+        rs = await self._tg_request('createForumTopic', True, **dic)
+        if not rs.get('ok'):
+            if rs.get('error_code') == 429:
+                raise ToMenyRequests(rs.get('description'), rs)
+            else:
+                raise TgAPIException(rs)
+        return rs
+
     async def get_user_profile_picture(self, user_id, **kwargs):
         dic = {
             'user_id': user_id
@@ -224,6 +313,7 @@ class Bot:
                     await _m(self.actions_from_cog.get(_m), command, message)
                 else:
                     await _m(command, message)
+
         if command in self.actions_from_cog:
             await command.func(self.actions_from_cog.get(command), message)
         else:
@@ -286,6 +376,16 @@ class Bot:
                     else:
                         await _m(message)
 
+    async def dispatch_chat_left_member(self, message):
+        print(self.listeners_handle.get('on_member_leave'))
+        if self.listeners_handle.get('on_member_leave'):
+            for _m in self.listeners_handle.get('on_member_leave'):
+                if _m in self.ignore_listener_filter or message.chat.id in self.chat_filter:
+                    if _m in self.actions_from_cog:
+                        await _m(self.actions_from_cog.get(_m), message)
+                    else:
+                        await _m(message)
+
     async def dispatch_chat_filter_error(self, message):
         if self.listeners_handle.get('on_chat_filter'):
             for _m in self.listeners_handle.get('on_chat_filter'):
@@ -354,21 +454,21 @@ class Bot:
             ms.pop(0)
             rs, c, from_aliases = self.search(' '.join(ms))
             if rs:
+                for i in range(c):
+                    ms.pop(0)
+                message.text = ' '.join(ms)
+                setattr(message, 'texts', ms)
+                if len(self.chat_filter) > 0:
+                    if message.chat.id not in self.chat_filter and rs not in self.ignore_filter:
+                        return await self.dispatch_chat_filter_error(message)
                 try:
-                    for i in range(c):
-                        ms.pop(0)
-                    message.text = ' '.join(ms)
-                    setattr(message, 'texts', ms)
-                    if len(self.chat_filter) > 0:
-                        if message.chat.id not in self.chat_filter and rs not in self.ignore_filter:
-                            return await self.dispatch_chat_filter_error(message)
                     await self.dispatch_command(message, rs)
                 except CallbackError as e:
                     await message.reply(e.message)
                     return await self.dispatch_error_command_invoke(message, rs, e)
-                except Exception as t:
+                except Exception as e:
+                    await self.dispatch_error_command_invoke(message, rs, e)
                     traceback.print_exc()
-                    return await self.dispatch_error_command_invoke(message, rs, t)
             else:
                 await self.dispatch_uknow_command(message)
         await self.dispatch_message(message)
@@ -378,6 +478,8 @@ class Bot:
             await self.dispatch_sticker(message)
         elif message.new_chat_member or message.new_chat_participant:
             await self.dispatch_new_member(message)
+        elif message.left_chat_member or message.left_chat_participant:
+            await self.dispatch_chat_left_member(message)
 
     async def longpoll(self):
         data = {}
